using System;
using System.IO;
using TopCoder.Util.CommandLine;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// The command line entry point of the File Statistics utility.
    /// It uses <see cref="ExtendedFileStatistics"/> to fulfill the task.
    /// When argument is invalid help message should be printed.
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safety.
    /// </thread-safety>
    ///
    /// <remarks>
    /// CommandLine [-c config-file] [-n namespace] [-b base-path] [-r]
    ///  [-o output-file] path [path2 path3 ...]
    /// -c config-file: provides a config file to initialize from
    /// -n namespace: provides a namespace to initialize from
    ///
    /// If neither is provided the default namespace is used.
    /// Provided namespace should be preloaded.
    /// -b base-path: all path will be relative to it.
    /// -r: whether to recursively traverse sub-directory.
    /// -o output-file: the output filename. If missing output is printed to console.
    /// </remarks>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class CommandLine
    {
        /// <summary>
        /// The namespace to used if no namespace is provided.
        /// </summary>
        public static readonly string Namespace = "TopCoder.File.Statistics.Main";

        /// <summary>
        /// Represents the config switch name.
        /// </summary>
        private const string COMAND_CONFIG = "c";

        /// <summary>
        /// Represents the namespace switch name.
        /// </summary>
        private const string COMAND_NAMESPACE = "n";

        /// <summary>
        /// Represents the base path switch name.
        /// </summary>
        private const string COMAND_BASE_PATH = "b";

        /// <summary>
        /// Represents the recursive switch name.
        /// </summary>
        private const string COMAND_RECURSIVE = "r";

        /// <summary>
        /// Represents the output file swith name.
        /// </summary>
        private const string COMAND_OUTPUT_FILE = "o";

        /// <summary>
        /// Private constructor to disallow instantiation.
        /// </summary>
        private CommandLine()
        {
        }

        /// <summary>
        /// Print help messages to console.
        /// </summary>
        private static void PrintHelp()
        {
            Console.WriteLine("FileStatisticsMain [-c:config-file] [-n:namespace] [-b:base-path] "
                + "[-r] [-o:output-file] path [path2 path3 ...]");
            Console.WriteLine("  -c:config-file    provides a config file to initialize from");
            Console.WriteLine("  -n:namespace      provides a namespace to initialize from");
            Console.WriteLine("  -b:base-path      all path will be relative to it");
            Console.WriteLine("  -r                whether to recursively traverse sub-directory");
            Console.WriteLine("  -o:output-file    the output filename. "
                + "If missing output is printed to console.");
            Console.WriteLine();
            Console.WriteLine("Notes:");
            Console.WriteLine("    If neither config-file nor namespace is provided the default "
                + "namespace is used.");
            Console.WriteLine("    Provided namespace should be preloaded.");
        }

        /// <summary>
        /// Command line entry. It uses <see cref="ExtendedFileStatistics"/> to fulfill the task.
        /// When argument is invalid help message should be printed.
        /// </summary>
        ///
        /// <param name="args">the command line args</param>
        public static void Main(string[] args)
        {
            try
            {
                // creat the switches
                CommandLineSwitch configSwitch = new CommandLineSwitch(
                    COMAND_CONFIG, "provides a config file to initialize from");
                CommandLineSwitch nameSpaceSwitch = new CommandLineSwitch(
                    COMAND_NAMESPACE, "provides a namespace to initialize from.");
                CommandLineSwitch basePathSwitch = new CommandLineSwitch(
                    COMAND_BASE_PATH, "all path will be relative to it.");
                CommandLineSwitch recursiveSwitch = new CommandLineSwitch(
                    COMAND_RECURSIVE, "whether to recursively traverse sub-directory.");
                CommandLineSwitch outputFileSwitch = new CommandLineSwitch(
                    COMAND_OUTPUT_FILE, "the output filename. If missing output is printed to console.");

                // create command line parser and add the switches
                CommandLineParser parser = new CommandLineParser();
                parser.AddSwitch(configSwitch);
                parser.AddSwitch(nameSpaceSwitch);
                parser.AddSwitch(basePathSwitch);
                parser.AddSwitch(recursiveSwitch);
                parser.AddSwitch(outputFileSwitch);

                // parse command
                parser.Parse(args);

                // if config-file switch supplied, load the config file
                if (parser.InputtedSwitches.Contains(COMAND_CONFIG))
                {
                    ConfigManager.GetInstance().LoadFile((string) configSwitch.Result[0]);
                }

                // if namespace switch supplied, use the supplied namespace
                // otherwise use default namespace
                string nameSpace = parser.InputtedSwitches.Contains(COMAND_NAMESPACE) ?
                    (string) nameSpaceSwitch.Result[0] : Namespace;

                // if recursive switch supplied, set to true,
                // otherwise set to false
                bool recursive = parser.InputtedSwitches.Contains(COMAND_RECURSIVE);

                // if base path switch is supplied, save it,
                // otherwise set to null.
                string basePath = parser.InputtedSwitches.Contains(COMAND_BASE_PATH) ?
                    (string) basePathSwitch.Result[0] : null;

                // if output file switch is upplied, save it
                // otherwise set to null.
                string outputFile = parser.InputtedSwitches.Contains(COMAND_OUTPUT_FILE) ?
                    (string) outputFileSwitch.Result[0] : null;

                // get the paths
                string[] paths = parser.Arguments;
                if (paths.Length == 0)
                {
                    PrintHelp();
                }
                else
                {
                    Process(nameSpace, recursive, basePath, outputFile, paths);
                }
            }
            catch(Exception)
            {
                // if any exception throw, show usage
                PrintHelp();
            }
        }

        /// <summary>
        /// Process the files in the supplied paths.
        /// </summary>
        ///
        /// <param name="nameSpace">the namespace.</param>
        /// <param name="recursive">recursive or not.</param>
        /// <param name="basePath">the base path</param>
        /// <param name="outputFile">the output file</param>
        /// <param name="paths">the paths to process.</param>
        private static void Process(string nameSpace, bool recursive, string basePath,
            string outputFile, string[] paths)
        {
            // use ExtendedFileStatistics to process files
            ExtendedFileStatistics statistics = new ExtendedFileStatistics(nameSpace);

            // if base path is not null, reset the base path.
            if (basePath != null)
            {
                statistics.Reporting.Reset(new DirectoryInfo(basePath));
            }

            // process files in each path
            foreach (string path in paths)
            {
                DirectoryInfo dir = new DirectoryInfo(path);
                if (dir.Exists)
                {
                    statistics.ProcessDirectory(dir, recursive);
                }
                else
                {
                    FileInfo file = new FileInfo(path);
                    if (file.Exists)
                    {
                        statistics.ProcessFile(file);
                    }
                    else
                    {
                        // path is not valid, just throw
                        // Exception to show help message,
                        // for it is required according to document.
                        throw new Exception();
                    }
                }
            }

            // if the output file is not null, print report to the file
            // otherwise print reprot to console
            if (outputFile != null)
            {
                statistics.PrintReport(new FileInfo(outputFile));
            }
            else
            {
                statistics.PrintReport();
            }
        }
    }
}