using System;
using System.IO;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileTypeMatcher"/> is a template class that handles file type oriented
    /// matching logic. File type is resolved by the Magic Numbers component.
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public abstract class FileTypeMatcher : IFileMatcher
    {
        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected FileTypeMatcher()
        {
        }

        /// <summary>
        /// It obtains the file type from the Magic Numbers component and uses the template method
        /// to decide whether the file matches. The proper configuration of Magic Numbers is not
        /// within the responsibility of this class. Any issue around this is translated into
        /// <see cref="FileMatchingException"/>. If the file does not have a correspondant file type,
        /// this would result in a false.
        /// </summary>
        ///
        /// <param name="file">the file to match</param>
        /// <returns>whether the file matches</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="FileMatchingException">if error occurs</exception>
        public bool Match(FileInfo file)
        {
            // verify file is valid
            FileStatisticsHelper.CheckNull(file, "file");

            try
            {
                MagicNumber magic = new MagicNumber();
                FileType type = magic.DetermineFileType(file);

                // If the file does not have a correspondant file type,
                // return false
                return (type == null) ? false : Match(type);
            }
            catch(FileMatchingException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new FileMatchingException("Error occured when match the file.", e);
            }
        }

        /// <summary>
        /// Template method that matches the file with file type result.
        /// </summary>
        ///
        /// <param name="type">the file type to match</param>
        /// <returns>whether the file matches</returns>
        ///
        /// <exception cref="FileMatchingException">if error occurs</exception>
        protected abstract bool Match(FileType type);
    }
}