using System;
using System.Collections;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// <see cref="StatsCollection"/> is a generic container for mapping from string keys
    /// to <see cref="StatsResult"/> instances. It is used in <see cref="AbstractStatsReporting"/>
    /// to hold file, directory and alias classified mappings.
    /// The class provides basic manipulation methods for the mapping.
    ///
    /// <thread-safety>
    /// This class is not thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class StatsCollection
    {
        /// <summary>
        /// Mapping from string to <see cref="StatsResult"/>. This is used to classify file,
        /// dir and alias basd stats in <see cref="AbstractStatsReporting"/>.
        /// The reference never change.
        /// </summary>
        private IDictionary stats = new Hashtable();

        /// <summary>
        /// Get the key set of the collection.
        /// Should return a copy and sort the strings in alphabetical order (case insensitive).
        /// </summary>
        ///
        /// <value>Represents the sorted key set of the collection.</value>
        public IList AllKeys
        {
            get
            {
                IList allKeys = new ArrayList(stats.Keys);
                ((ArrayList)allKeys).Sort();

                return allKeys;
            }
        }

        /// <summary>
        /// Get the stats result associated with a key. The result is cloned before return.
        /// If no result is associated return null.
        /// </summary>
        ///
        /// <param name="key">the key to retrieve</param>
        /// <returns>the result associated with key</returns>
        ///
        /// <exception cref="ArgumentNullException">if key is null</exception>
        /// <exception cref="ArgumentException">if key is empty after trimmed</exception>
        public StatsResult this[string key]
        {
            get
            {
                FileStatisticsHelper.CheckNullOrEmptyString(key, "key");

                // if the key is existed, return the cloned stats
                // otherwise return null.
                return this.stats.Contains(key) ? new StatsResult((StatsResult)this.stats[key]) : null;
            }
        }

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public StatsCollection()
        {
        }

        /// <summary>
        /// Add a result to specified key.
        /// </summary>
        ///
        /// <param name="key">the key to associate result with</param>
        /// <param name="stats">the result to add</param>
        ///
        /// <exception cref="ArgumentNullException">if key or stats is null</exception>
        /// <exception cref="ArgumentException">if key is empty after trimmed</exception>
        public void Add(string key, StatsResult stats)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(key, "key");
            FileStatisticsHelper.CheckNull(stats, "stats");

            if (this.stats.Contains(key))
            {
                // the key already exists, should aggregate the result with this one.
                ((StatsResult) this.stats[key]).Aggregate(stats);
            }
            else
            {
                // save a clone to the mapping.
                this.stats.Add(key, new StatsResult(stats));
            }
        }

        /// <summary>
        /// Clear the collection.
        /// </summary>
        public void Clear()
        {
            this.stats.Clear();
        }
    }
}