using System;
using System.IO;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// <see cref="XmlStatsReporting"/> is a reporting implementation that renders XML reports.
    /// Refer to component specification for formatting of the XML. It is capable of using a
    /// stylesheet to transform the XML into user-oriented format (HTML, CSV, etc.)
    ///
    /// <configuration>
    /// It supports a single property "stylesheet" which contains a filename of the stylesheet.
    /// This property is optional. If it's missing the raw XML is generated.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is not thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class XmlStatsReporting : AbstractStatsReporting
    {
        /// <summary>
        /// The property which specifies a stylesheet to apply on the xml.
        /// If this property is not specified, the raw output is generated.
        /// </summary>
        public static readonly string StylesheetPropertyKey = "stylesheet";

        /// <summary>
        /// Represents the report xml file element 'report'.
        /// </summary>
        private const string TAG_REPORT = "report";

        /// <summary>
        /// Represents the report xml file element 'files'.
        /// </summary>
        private const string TAG_FILES = "files";

        /// <summary>
        /// Represents the report xml file element name 'directories'.
        /// </summary>
        private const string TAG_DIRECTORIES = "directories";

        /// <summary>
        /// Represents the report xml file element name 'aliases'.
        /// </summary>
        private const string TAG_ALIASES = "aliases";

        /// <summary>
        /// Represents the report xml file element name 'overall'.
        /// </summary>
        private const string TAG_OVERALL = "overall";

        /// <summary>
        /// Represents the report xml file element name 'file'.
        /// </summary>
        private const string TAG_FILE = "file";

        /// <summary>
        /// Represents the report xml file element name 'directory'.
        /// </summary>
        private const string TAG_DIRECTORY = "directory";

        /// <summary>
        /// Represents the report xml file element name 'alias'.
        /// </summary>
        private const string TAG_ALIAS = "alias";

        /// <summary>
        /// Represents the report xml file element attribute  'name'.
        /// </summary>
        private const string ATTRIBUTE_NAME = "name";

        /// <summary>
        /// The stylesheet to apply on the raw xml generated. This is initialized in constructor
        /// and never changed. The value can be null indicating no stylesheet is used.
        /// </summary>
        private FileInfo stylesheet = null;

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public XmlStatsReporting()
        {
        }

        /// <summary>
        /// Constructor with a styleshee. Simply store it to member variable.
        /// </summary>
        ///
        /// <param name="stylesheet">the stylesheet</param>
        ///
        /// <exception cref="ArgumentNullException">if stylesheet is null</exception>
        /// <exception cref="ArgumentException">if stylesheet is not a valid file</exception>
        public XmlStatsReporting(FileInfo stylesheet)
        {
            FileStatisticsHelper.CheckNull(stylesheet, "stylesheet");
            if (!stylesheet.Exists)
            {
                throw new ArgumentException("the '" + stylesheet + "' is not a valid file.");
            }

            this.stylesheet = stylesheet;
        }

        /// <summary>
        /// Configures from a property. propagate it to super constructor. After that, it reads
        /// a String property from "stylesheet" and store it to variable. This property is optional,
        /// but should be an existing filename if exists.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        public XmlStatsReporting(string nameSpace, string prefix)
        {
            // verify the namespace is null or empty string
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            // get the property name
            string property = (prefix.Trim().Length == 0) ?
            StylesheetPropertyKey : prefix + "." + StylesheetPropertyKey;

            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get the extensions from the config file
                string propertyValue = cm.GetValue(nameSpace, property);
                if (propertyValue != null)
                {
                    this.stylesheet = new FileInfo(propertyValue);
                    if (!this.stylesheet.Exists)
                    {
                        throw new ConfigurationException("the '" + propertyValue + "' is not a valid file.");
                    }
                }
            }
            catch (ConfigurationException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ConfigurationException(
                    "Failed to get the correct stylesheet from the config file.", e);
            }
        }

        /// <summary>
        /// This generates an XML format report if no stylesheet is specified,
        /// or outputs the result of XSLT.
        /// </summary>
        ///
        /// <returns>the generated report</returns>
        ///
        /// <exception cref="StatsReportingException">if rendering fails</exception>
        public override string GenerateReport()
        {
            try
            {
                XmlDocument doc = new XmlDocument();

                // create the root node
                XmlNode root = CreateXmlNode(doc, TAG_REPORT);

                // create the files node
                XmlNode files = CreateXmlNode(doc, TAG_FILES);
                AddStatsCollectionNodes(files, doc, this.FileStats, TAG_FILE);

                // create the directories node
                XmlNode directories = CreateXmlNode(doc, TAG_DIRECTORIES);
                AddStatsCollectionNodes(directories, doc, this.DirectoryStats, TAG_DIRECTORY);

                // create the aliases nodes
                XmlNode aliases = CreateXmlNode(doc, TAG_ALIASES);
                AddStatsCollectionNodes(aliases, doc, this.AliasStats, TAG_ALIAS);

                // create the overal nodes
                XmlNode overall = CreateXmlNode(doc, TAG_OVERALL);
                AddStasResultNodes(overall, doc, this.OverallStats);

                // add the create nodes to document
                root.AppendChild(files);
                root.AppendChild(directories);
                root.AppendChild(aliases);
                root.AppendChild(overall);

                doc.AppendChild(root);

                return GetReportContent(doc);
            }
            catch(Exception e)
            {
                throw new StatsReportingException("Failed to generate report.", e);
            }
        }

        /// <summary>
        /// Create an xml node with the supplied name
        /// </summary>
        ///
        /// <param name="doc">the doc owned the node</param>
        /// <param name="nodeName">the node name</param>
        /// <returns>the created node</returns>
        private XmlNode CreateXmlNode(XmlDocument doc, string nodeName)
        {
            return doc.CreateNode(XmlNodeType.Element, nodeName, string.Empty);
        }

        /// <summary>
        /// Create xml nodes for stats result.
        /// </summary>
        ///
        /// <param name="parent">the parent node for the stats node.</param>
        /// <param name="doc">the xml document.</param>
        /// <param name="result">the result containing metrics to create nodes.</param>
        private void AddStasResultNodes(XmlNode parent, XmlDocument doc, StatsResult result)
        {
            foreach (Metric metric in result.Metrics)
            {
                AddMetricNode(parent, doc, metric, result[metric]);
            }
        }

        /// <summary>
        /// Create xml nodes for stats collection.
        /// </summary>
        ///
        /// <param name="parent">the parent node for the stats node.</param>
        /// <param name="doc">the xml document.</param>
        /// <param name="stats">the stats collection to create nodes</param>
        /// <param name="statsName">the stats display name.</param>
        private void AddStatsCollectionNodes(XmlNode parent, XmlDocument doc,
            StatsCollection stats, string statsName)
        {
            // add each stats result in the stas
            foreach (string key in stats.AllKeys)
            {
                // create the stats node and set the name attribute.
                XmlNode statsNode = CreateXmlNode(doc, statsName);
                XmlAttribute attr = doc.CreateAttribute(string.Empty, ATTRIBUTE_NAME, string.Empty);
                attr.Value = key;
                statsNode.Attributes.Append(attr);

                // add each metric in the statsResult
                AddStasResultNodes(statsNode, doc, stats[key]);
                parent.AppendChild(statsNode);
            }
        }

        /// <summary>
        /// Create xml node for metric.
        /// </summary>
        ///
        /// <param name="parent">the parent node for the metric node.</param>
        /// <param name="doc">the xml document.</param>
        /// <param name="metric">the metric to create node</param>
        /// <param name="metricValue">the metric value</param>
        private void AddMetricNode(XmlNode parent, XmlDocument doc, Metric metric, long metricValue)
        {
            // create node with metric's display name as tag
            // and set the text.
            XmlNode child = doc.CreateNode(XmlNodeType.Element, metric.DisplayName, string.Empty);
            XmlNode text = doc.CreateNode(XmlNodeType.Text, string.Empty, string.Empty);
            text.Value = metricValue.ToString();

            // append the create node to parent
            child.AppendChild(text);
            parent.AppendChild(child);
        }

        /// <summary>
        /// Get the report contents, if not stylesheet, just return raw xml,
        /// othwerwise, transform the xml.
        /// </summary>
        ///
        /// <param name="doc">the xml document.</param>
        /// <returns>the report content.</returns>
        private string GetReportContent(XmlDocument doc)
        {
            if (stylesheet != null)
            {
                MemoryStream output = new MemoryStream();

                // transfrom the xml using the stylesheet
                XslTransform xslt = new XslTransform();
                xslt.Load(stylesheet.FullName, new XmlUrlResolver());
                xslt.Transform(new XPathDocument(new StringReader(doc.InnerXml)),
                    null, new StreamWriter(output), null);

                // return the report
                output.Flush();
                output.Seek(0, SeekOrigin.Begin);
                return new StreamReader(output).ReadToEnd();
            }
            else
            {
                // return the report
                return doc.InnerXml;
            }
        }
    }
}