using System;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <see cref="StatsResult"/> is container for multiple results, each of which are mapped
    /// from a Metric to a Long. This class wraps some map operations as well as provides a way
    /// to clone and aggregate results.
    ///
    /// <thread-safety>
    /// This class is not thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class StatsResult
    {
        /// <summary>
        /// The results mapping from metric to the statistical quantity (Long).
        /// This reference is never changed.
        /// The key is the metric, and the value is the statistical quantity (Long).
        /// </summary>
        private IDictionary results = new Hashtable();

        /// <summary>
        /// Get a list of all the associated metrics. Should return a copy from inner collection.
        /// </summary>
        ///
        /// <value>Represents the list of all the associated metrics.</value>
        public IList Metrics
        {
            get
            {
                return new ArrayList(this.results.Keys);
            }
        }

        /// <summary>
        /// Set or get a certain stat (associate metric with value).
        /// This method will overwrite old values associated.
        /// </summary>
        ///
        /// <param name="metric">the metric to associate with</param>
        /// <returns>the quantity</returns>
        ///
        /// <exception cref="ArgumentNullException">if metric is null</exception>
        /// <exception cref="ArgumentException">if no value associated</exception>
        public long this[Metric metric]
        {
            get
            {
                FileStatisticsHelper.CheckNull(metric, "metric");

                if (this.results.Contains(metric))
                {
                    // return the contained value
                    return (long) this.results[metric];
                }
                else
                {
                    // no value associated with the specific metric
                    throw new ArgumentException(
                        "no valued associated with the specific metric.", "metric");
                }
            }
            set
            {
                FileStatisticsHelper.CheckNull(metric, "metric");

                if (this.results.Contains(metric))
                {
                    this.results[metric] = value;
                }
                else
                {
                    this.results.Add(metric, value);
                }
            }
        }

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public StatsResult()
        {
        }

        /// <summary>
        /// Create a result based on another result. This acts as a way to clone the instance.
        /// </summary>
        ///
        /// <param name="result">the result to clone from.</param>
        ///
        /// <exception cref="ArgumentNullException">if result is null</exception>
        public StatsResult(StatsResult result)
        {
            // clone the instance, just aggregate the result
            Aggregate(result);
        }

        /// <summary>
        /// Aggregate the result into the current result.
        /// </summary>
        ///
        /// <param name="result">the result to aggregate from</param>
        ///
        /// <exception cref="ArgumentNullException">if result is null</exception>
        public void Aggregate(StatsResult result)
        {
            FileStatisticsHelper.CheckNull(result, "result");

            foreach (Metric metric in result.results.Keys)
            {
                if (this.results.Contains(metric))
                {
                    // aggregate each metric into this statsResult
                    this.results[metric] = ((long) this.results[metric]) + result[metric];
                }
                else
                {
                    // add the metric
                    this.results.Add(metric, result[metric]);
                }
            }
        }
    }
}