using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for <see cref="RegexMatcher"/> class.
    /// It mainly tests the following:
    /// static member: PatternsPropertyKey
    /// constructor: RegexMatcher(string), RegexMatcher(IList), RegexMatcher(string, string)
    /// member: Match(string)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class RegexMatcherUnitTest
    {
        /// <summary>
        /// Set up for each test.
        /// Load config file.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Tear down for each test.
        /// Remove config files.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }


        /// <summary>
        /// Test static member PatternsPropertyKey, it should be 'patterns'.
        /// </summary>
        [Test]
        public void TestPatternsPropertyKey()
        {
            Assert.AreEqual("patterns", RegexMatcher.PatternsPropertyKey,
                "PatternsPropertyKey of RegexMatcher should be 'patterns'.");
        }

        /// <summary>
        /// Test ctor RegexMatcher(Regex pattern), with pattern is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_PatternIsNull()
        {
            new RegexMatcher((Regex) null);
        }

        /// <summary>
        /// Test ctor RegexMatcher(Regex pattern), with pattern is valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Regex pattern = new Regex("t[xyz]t");
            Assert.IsNotNull(new RegexMatcher(pattern), "Failed to create instance of RegexMatcher.");
        }

        /// <summary>
        /// Test ctor RegexMatcher(IList patterns), with patterns is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PatternsIsNull()
        {
            new RegexMatcher((IList) null);
        }

        /// <summary>
        /// Test ctor RegexMatcher(IList patterns), with patterns is empty list,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_PatternsIsEmpty()
        {
            new RegexMatcher(new ArrayList());
        }

        /// <summary>
        /// Test ctor RegexMatcher(IList patterns), with patterns contains null.
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_PatternsContainsNull()
        {
            IList patterns = new ArrayList();
            patterns.Add(null);
            patterns.Add(new Regex("t[xyz]t"));

            new RegexMatcher(patterns);
        }

        /// <summary>
        /// Test ctor RegexMatcher(IList patterns), with patterns contains non-Regex instance.
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_PatternsContainsNonRegex()
        {
            IList patterns = new ArrayList();
            patterns.Add(new object());
            patterns.Add(new Regex("t[xyz]t"));

            new RegexMatcher(patterns);
        }

        /// <summary>
        /// Test ctor RegexMatcher(IList patterns), with patterns contains some Regex instances.
        /// An instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            IList patterns = new ArrayList();
            patterns.Add(new Regex("pl"));
            patterns.Add(new Regex("t[xyz]t"));

            new RegexMatcher(patterns);
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_NamespaceIsNull()
        {
            new RegexMatcher(null, "processor3.classname");
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with nameSpace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor3_NamespaceIsEmpty()
        {
            new RegexMatcher("  ", "processor3.classname");
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_PrefixIsNull()
        {
            new RegexMatcher(UnitTestHelper.DEFAULT_NAMESPACE, null);
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError1()
        {
            new RegexMatcher(UnitTestHelper.INVALID_NAMESPACE, "RegexMatcher1");
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError2()
        {
            new RegexMatcher(UnitTestHelper.INVALID_NAMESPACE, "RegexMatcher2");
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError3()
        {
            new RegexMatcher(UnitTestHelper.INVALID_NAMESPACE, "RegexMatcher3");
        }

        /// <summary>
        /// Test ctor RegexMatcher(string nameSpace, string prefix), with nameSpace and prefix is valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor3()
        {
            IFileMatcher matcher = new RegexMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor3.matcher");
            Assert.IsNotNull(matcher, "Failed to create instance of RegexMatcher");
        }

        /// <summary>
        /// Test Match(string filename), since it is protected, test Match(FileInfo file) instead.
        /// the default config file suported two patterns "[cC](#|\+\+)?" and "[jJ]ava".
        /// </summary>
        [Test]
        public void TestMatch_True()
        {
            IFileMatcher matcher = new RegexMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor3.matcher");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.c")),
                "c file should be matched.");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.cs")),
                "cs file should be matched.");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.java")),
                "java file should be matched.");
        }

        /// <summary>
        /// Test Match(string filename), since it is protected, test Match(FileInfo file) instead.
        /// the default config file suported two patterns "[cC](#|\+\+)?" and "[jJ]ava".
        /// </summary>
        [Test]
        public void TestMatch_False()
        {
            IFileMatcher matcher = new RegexMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor3.matcher");
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.pl")),
                "pl file should be not matched.");
        }
    }
}