using System;
using System.IO;
using System.Collections;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// Unit Test for <see cref="StatsCollection"/> class.
    /// It mainly tests the following:
    /// constructor: StatsCollection(),
    /// property: AllKeys
    /// member: Add(string, StatsResult), Clear(), indexer,
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class StatsCollectionUnitTest
    {
        /// <summary>
        /// Represents instance of <see cref="StatsCollection"/>.
        /// </summary>
        private StatsCollection stats = null;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            stats = new StatsCollection();
        }

        /// <summary>
        /// Test ctor StatsCollection(), instance should be created.
        /// </summary>
        [Test]
        public void TestCtor()
        {
            Assert.IsNotNull(stats, "Failed to create instance of StatsCollection.");
        }

        /// <summary>
        /// Test Add(string key, StatsResult stats), with key is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAdd_KeyIsNull()
        {
            stats.Add(null, new StatsResult());
        }

        /// <summary>
        /// Test Add(string key, StatsResult stats), with key is emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestAdd_KeyIsEmpty()
        {
            stats.Add("  ", new StatsResult());
        }

        /// <summary>
        /// Test Add(string key, StatsResult stats), with stats is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAdd_StatsIsNull()
        {
            stats.Add("Test", null);
        }

        /// <summary>
        /// Test Add(string key, StatsResult stats), with key not existed,
        /// the new key should be added.
        /// </summary>
        [Test]
        public void TestAdd1()
        {
            StatsResult result = new StatsResult();
            stats.Add("Test", result);
            Assert.AreEqual(1, stats.AllKeys.Count, "Failed to add key");
            Assert.AreEqual("Test", stats.AllKeys[0], "Failed to add key");
            Assert.AreEqual(result.Metrics.Count, stats["Test"].Metrics.Count, "Failed to add key");
        }

        /// <summary>
        /// Test Add(string key, StatsResult stats), with key already existed,
        /// the result should be aggregated.
        /// </summary>
        [Test]
        public void TestAdd2()
        {
            // create some metrics
            Metric metric1 = new Metric("1", "1", "1");
            Metric metric2 = new Metric("2", "2", "2");

            // create a stats result and add
            StatsResult result1 = new StatsResult();
            result1[metric1] = 10;
            stats.Add("Test", result1);

            // verify the add
            Assert.AreEqual(1, stats.AllKeys.Count, "Failed to add key");
            Assert.AreEqual("Test", stats.AllKeys[0], "Failed to add key");
            Assert.AreEqual(10, stats["Test"][metric1], "Failed to add key");

            // create anoth stats result and add
            StatsResult result2 = new StatsResult();
            result2[metric1] = 10;
            result2[metric2] = 20;
            stats.Add("Test", result2);

            // verify the add
            Assert.AreEqual(1, stats.AllKeys.Count, "Failed to add key");
            Assert.AreEqual("Test", stats.AllKeys[0], "Failed to add key");
            Assert.AreEqual(20, stats["Test"][metric1], "Failed to add key");
            Assert.AreEqual(20, stats["Test"][metric2], "Failed to add key");
        }

        /// <summary>
        /// Test Clear(), all stats should be cleared.
        /// </summary>
        [Test]
        public void TestClear()
        {
            // add some stats result
            stats.Add("Test1", new StatsResult());
            stats.Add("Test2", new StatsResult());
            stats.Add("Test3", new StatsResult());
            Assert.AreEqual(3, stats.AllKeys.Count, "Failed t add the stats result");

            // clear the result
            stats.Clear();

            // verify the clear
            Assert.AreEqual(0, stats.AllKeys.Count, "Failed t clear the stats result");
        }

        /// <summary>
        /// Test property AllKeys, with no statsresult is added, empty list is expected.
        /// </summary>
        [Test]
        public void TestAllKeys_Empty()
        {
            Assert.AreEqual(0, stats.AllKeys.Count, "No stats result is added, empty list is expected.");
        }

        /// <summary>
        /// Test property AllKeys, add the keys of added should be returned by sort.
        /// </summary>
        [Test]
        public void TestAllKeys()
        {
            // add some stats result not sorted
            stats.Add("Test2", new StatsResult());
            stats.Add("Test1", new StatsResult());
            stats.Add("Test4", new StatsResult());
            stats.Add("Test3", new StatsResult());

            IList keys = stats.AllKeys;
            Assert.AreEqual(4, keys.Count, "Failed to get the correct keys list.");
            for (int i = 1; i <= 4;++i)
            {
                Assert.AreEqual("Test" + i, keys[i - 1], "Failed to get the correct keys list.");
            }
        }

        /// <summary>
        /// Test indexer this[string key], with key is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexer_KeyIsNull()
        {
            StatsResult resutl = stats[null];
        }

        /// <summary>
        /// Test indexer this[string key], with key is empty string, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestIndexer_KeyIsEmpty()
        {
            StatsResult resutl = stats["  "];
        }

        /// <summary>
        /// Test indexer this[string key], with key is not existed, null is expected.
        /// </summary>
        [Test]
        public void TestIndexer_Getter_KeyNotExist()
        {
            Assert.IsNull(stats["NotExisted"], "No such key existed, null is expected.");
        }

        /// <summary>
        /// Test indexer this[string key], with key is existed, the mapped StatsResult should be returned.
        /// </summary>
        [Test]
        public void TestIndexer_Getter()
        {
            // create some stats reulst
            Metric metric1 = new Metric("1", "1", "1");
            Metric metric2 = new Metric("2", "2", "2");
            StatsResult result1 = new StatsResult();
            result1[metric1] = 11;
            result1[metric2] = 12;
            StatsResult result2 = new StatsResult();
            result2[metric1] = 21;
            result2[metric2] = 22;

            // add result to stats
            stats.Add("key1", result1);
            stats.Add("key2", result2);

            // get the result by key
            StatsResult returned = stats["key1"];

            // verify the returned result
            Assert.AreEqual(11, returned[metric1],
                "The returned statsResult is not correct");
            Assert.AreEqual(12, returned[metric2],
                "The returned statsResult is not correct");
        }
    }
}