using System;
using TopCoder.Util.DataValidator;

namespace TopCoder.File.Statistics
{
	/// <summary>
	/// ArgumentOption is a pure data oriented class which packs together the siwtch name
	/// and help message, required flag and an optional validator for an option. 
	/// An IArgumentHandler can propose multiple options to the command line interface.
	/// 
	/// <thread-safety>
	/// This class is immutable and thread-safe.
	/// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0b</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
	public class ArgumentOption
	{
	    /// <summary>
	    /// The switch name for this option. 
	    /// Swith name unique identifies an argument. But switch name does not need
	    /// to be unique among all proposals because the arguments could be shared
	    /// (for instance we are very likely to configure a IFileMatcher twice).
	    /// </summary>
	    private string switchName;
	    
	    /// <summary>
	    /// This is the helper message to be printed to the user. 
	    /// It's free style text but should normally respect the format of
	    ///  the help message screen of the respective implementation.
	    /// </summary>
	    private string helpMessage;
	    
	    /// <summary>
	    /// A flag indicates whether the switch is required or not.
	    /// Notice it is only concerned with the switch rather than any actual value
	    /// after the switch.
	    /// </summary>
	    private bool required; 
	    
	    /// <summary>
	    /// An optional validator to validate the argument value at the command line
	    /// interface level. This validation is not required and IArgumentHandler
	    /// can perform more validation when the arguments are received.
	    /// </summary>
	    private IValidator validator;
	    
	    /// <summary>
	    /// Get the switch name.
	    /// </summary>
	    /// <value>Represents the switch name.</value>
	    public string SwitchName
	    {
	        get 
	        {
	            return this.switchName;
	        }
	    }
	   
	    /// <summary>
	    /// Get the help message.
	    /// </summary>
	    /// <value>Represents the help message.</value>
	    public string HelpMessage
	    {
	        get
	        {
	            return this.helpMessage;
	        }
	    }
	    
	    /// <summary>
	    /// Get the required flag.
	    /// </summary>
	    /// <value>Represents the required flag.</value>
	    public bool IsRequired
	    {
	        get
	        {
	            return this.required;
	        }
	    }
	    
	    /// <summary>
	    /// Get the object validator.
	    /// </summary>
	    /// <value>Represents the object validator.</value>
	    public IValidator Validator
	    {
	        get
	        {
	            return this.validator;
	        }
	    }
	    	
	    /// <summary>
	    /// Constructor with 3 of the arguments (with validator set to null).
	    /// </summary>
	    /// 
	    /// <param name="switchName">the switch name</param>
	    /// <param name="helpMessage">the help message</param>
	    /// <param name="required">the required flag</param>
	    /// <exception cref="ArgumentNullException" >
	    /// if switchName or helpMessage is null</exception>
	    /// <exception cref="ArgumentException" >
	    /// if switchName or helpMessage is empty after trimmed</exception>
	    public ArgumentOption(string switchName, string helpMessage, bool required)
	        : this(switchName, helpMessage, required, null)
	    {
	        FileStatisticsHelper.CheckNullOrEmptyString(switchName, "switchName");
	        FileStatisticsHelper.CheckNullOrEmptyString(helpMessage, "helpMessage");
	        
	        this.switchName = switchName;
	        this.helpMessage = helpMessage;
	        this.required = required;
	    }
	    
	    /// <summary>
	    /// Constructor with all arguments.
	    /// </summary>
	    /// 
	    /// <param name="switchName">the switch name</param>
	    /// <param name="helpMessage">the help message</param>
	    /// <param name="required">the required flag</param>
	    /// <param name="validator">the object validator</param>
	    /// <exception cref="ArgumentNullException" >
	    /// if switchName or helpMessage or validator is null</exception>
	    /// <exception cref="ArgumentException" >
	    /// if switchName or helpMessage is empty after trimmed</exception>
	    public ArgumentOption(string switchName, string helpMessage, bool required, IValidator validator)
	        : this(switchName, helpMessage, required)
	    {
	        FileStatisticsHelper.CheckNull(validator, "validator");
	        
	        this.validator = validator;
	    }
	}	    
}
