using System;
using System.IO;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <see cref="IFileProcessor"/> defines the contract to process a file.
    /// One file processor defines an algorithm to produce statistical result for a certain class
    /// of files against certain metrics. (For instance count line numbers for a text file.)
    /// Multiple <see cref="IFileProcessor"/>s are aggregated into <see cref="FileStatistics"/>,
    /// which can run in two modes (process with first matching processor and process with each
    /// matching processor). File processors are mapped with an alias. This alias is dynamically
    /// provided by the user and the reporting can be grouped against this alias.
    ///
    /// <contract-constructor>
    /// All implementation should have a constructor with two string as arguments
    /// in order to reflect from configuration file.
    /// This requirement can be dropped if it's not necessary to use this way.
    /// </contract-constructor>
    ///
    /// <thread-safety>
    /// Implementations are advised to be stateless and thread-safe. But it's not a requirement.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public interface IFileProcessor
    {
        /// <summary>
        /// Get the alias of the processor. An alias is the official name of the processor
        /// which is also used to classify the stats result against. This should be non-empty.
        /// </summary>
        ///
        /// <value>Represents the alias of the processor.</value>
        string Alias
        {
            get;
        }

        /// <summary>
        /// Get the file matcher associated with the processor in order to determine
        /// whether a file can/should be processed with the processor.
        /// </summary>
        ///
        /// <value>Represents the file matcher associated with the processor.</value>
        IFileMatcher FileMatcher
        {
            get;
        }

        /// <summary>
        /// Get a list of all the supported metrics by this processor.
        /// This method is mostly for end user.
        /// It does not fulfill any responsibility in the current framework.
        /// </summary>
        ///
        /// <value>Represents the list of the supported metrics.</value>
        IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Process the file and generate the result. A new result should be generated each time,
        /// whose keys are the metrics returned from <see cref="SupportedMetrics"/>.
        /// </summary>
        ///
        /// <param name="file">the file to process.</param>
        /// <returns>the stats result.</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null.</exception>
        /// <exception cref="IOException">if io operation fails.</exception>
        /// <exception cref="FileProcessingException">if processor fails.</exception>
        StatsResult Process(FileInfo file);
    }
}