using System;
using System.Collections;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="MimeMatcher"/> concrete implementation that matches file types with
    /// one or more mimes (as configured in Magic Numbers).
    ///
    /// <configuration>
    /// This class can be constructed via reflection by the constructor contract of
    /// <see cref="IFileMatcher"/>.
    /// The supported property is "mimes" which is potentially multi-valued.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class MimeMatcher : FileTypeMatcher
    {
        /// <summary>
        /// Configuration key that contains multiple file mimes to match.
        /// Empty mimes are not allowed even if they are valid in the Magic Numbers component.
        /// </summary>
        public static readonly string MimesPropertyKey = "mimes";

        /// <summary>
        /// A list of file mimes (string) to match.
        /// This list is initialized in place and is immutable.
        /// </summary>
        private IList mimes = new ArrayList();

        /// <summary>
        /// Constructor with single file mime. Simply creates a list to hold it.
        /// </summary>
        ///
        /// <param name="mime">the file mime</param>
        ///
        /// <exception cref="ArgumentNullException">if mime is null</exception>
        /// <exception cref="ArgumentException">if mime is empty</exception>
        public MimeMatcher(string mime)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(mime, "mime");

            this.mimes.Add(mime);
        }

        /// <summary>
        /// Constructor with multiple file mimes. Simply save a clone of the list.
        /// </summary>
        ///
        /// <param name="mimes">the file mimes to match</param>
        ///
        /// <exception cref="ArgumentNullException">if mimes is null</exception>
        /// <exception cref="ArgumentException">if mimes is empty or contains
        /// non-String instance or contains empty string (after trimmed)</exception>
        public MimeMatcher(IList mimes)
        {
            // verify the mimes is valid
            FileStatisticsHelper.CheckNull(mimes, "mimes");

            // save the mimes
            FileStatisticsHelper.SaveListValue(this.mimes, mimes, "mimes", false);
        }

        /// <summary>
        /// Constructor that loads mimes from a property.
        /// It should look for an "mimes" sub-property and extract multiple values.
        /// </summary>
        ///
        /// <param name="nameSpace">non-empty, the namespace to load config from.</param>
        /// <param name="prefix">can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if property values does not contain any values
        /// or any value is empty after trimmed </exception>
        public MimeMatcher(string nameSpace, string prefix)
        {
            // verify the namespace is null or empty string
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            // get the property name
            string propertyName = (prefix == null) ?
                MimesPropertyKey : prefix + MimesPropertyKey;

            // load the config values
            FileStatisticsHelper.LoadConfigValues(nameSpace, propertyName, this.mimes, false);
        }

        /// <summary>
        /// Check whether any of the mimes match the type. The matching is case-sensitive.
        /// </summary>
        ///
        /// <param name="type">the type to match</param>
        /// <returns>whether the type matches</returns>
        protected override bool Match(FileType type)
        {
            return FileStatisticsHelper.Match(this.mimes, type.Mime);
        }
    }
}
