using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor.LineCounter;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Unit Test for <see cref="ExtendedFileStatistics"/> class.
    /// It mainly tests the following:
    /// constructor:ExtendedFileStatistics(), ExtendedFileStatistics(string)
    /// member: ProcessDirectory(DirectoryInfo),  ProcessDirectory(DirectoryInfo, bool),
    /// PrintReport(), PrintReport(FileInfo)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class ExtendedFileStatisticsUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="ExtendedFileStatistics"/>, used to test it's public member.
        /// </summary>
        private ExtendedFileStatistics statistics;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);

            statistics = new ExtendedFileStatistics();
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
        }

        /// <summary>
        /// Test ExtendedFileStatistics(), instance should be created.
        /// </summary>
        public void TestCtor1()
        {
            Assert.IsNotNull(new ExtendedFileStatistics(), "Failed to create ExtendedFileStatistics.");
        }

        /// <summary>
        /// Test ctor ExtendedFileStatistics(string nameSpace), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NameSpaceIsNull()
        {
            new ExtendedFileStatistics(null);
        }

        /// <summary>
        /// Test ctor ExtendedFileStatistics(string nameSpace), with nameSpace is null,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NameSpaceIsEmpty()
        {
            new ExtendedFileStatistics("  ");
        }

        /// <summary>
        /// Test ctor ExtendedFileStatistics(string nameSpace), nameSpace is loaded and correct,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Assert.IsNotNull(new ExtendedFileStatistics(UnitTestHelper.DEFAULT_NAMESPACE),
                "Failed to create instance of ExtendedFileStatistics.");
        }

        /// <summary>
        /// Test ProcessDirectory(DirectoryInfo directory), with directory is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessDirectory1_DirectoryIsNull()
        {
            statistics.ProcessDirectory(null);
        }

        /// <summary>
        /// Test ProcessDirectory(DirectoryInfo directory), with directory is Not existed,
        /// IOException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestProcessDirectory1_DirectoryNotExisted()
        {
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/notexisted"));
        }

        /// <summary>
        /// Test ProcessDirectory(DirectoryInfo directory), with files existed in directory
        /// </summary>
        [Test]
        public void TestProcessDirectory1()
        {
            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new CStyleLineCounter("C", new ExtensionMatcher("c"));

            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            statistics.AddFileProcessor(processor3);
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/Processor/LineCounter"));
        }

        /// <summary>
        /// Test ProcessDirectory(DirectoryInfo directory, bool recursive), with directory is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessDirectory2_DirectoryIsNull()
        {
            statistics.ProcessDirectory(null, false);
        }

        /// <summary>
        /// Test ProcessDirectory(DirectoryInfo directory, bool recursive), with directoryis Not existed,
        /// IOException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestProcessDirectory2_DirectoryNotExisted()
        {
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/notexisted"), false);
        }

        /// <summary>
        ///Test ProcessDirectory(DirectoryInfo directory, bool recursive), with directory contains sub-directory.
        /// </summary>
        [Test]
        public void TestProcessDirectory2()
        {
            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new CStyleLineCounter("C", new ExtensionMatcher("c"));

            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            statistics.AddFileProcessor(processor3);
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/Processor"), true);
        }

        /// <summary>
        /// Test PrintReport(), it output the report to the console.
        /// </summary>
        [Test]
        public void TestPrintReport1()
        {
            // add processors
            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new CStyleLineCounter("C", new ExtensionMatcher("c"));

            statistics.Reporting.Reset(new DirectoryInfo("../../test_files"));

            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            statistics.AddFileProcessor(processor3);
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/Processor"), true);

            // set the console output to new
            MemoryStream stream = new MemoryStream(2048);
            TextWriter newOut = new StreamWriter(stream);
            TextWriter oldOut = Console.Out;
            Console.SetOut(newOut);

            statistics.PrintReport();

            // restore the console output
            Console.SetOut(oldOut);

            // verify the output with the usage file
            newOut.Flush();
            stream.Seek(0, SeekOrigin.Begin);
            string content = new StreamReader(stream).ReadToEnd();

            UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/extended1.txt",
                "The report result is not expected.");
        }

        /// <summary>
        /// Test PrintReport(FileInfo file), with file is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestPrintReport2_FileIsNull()
        {
            statistics.PrintReport(null);
        }

        /// <summary>
        /// Test PrintReport(FileInfo file), with file is can't be created.
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestPrintReport2_FileCreatedFailed()
        {
            statistics.PrintReport(new FileInfo("../../test_files/notexisted/notexisted.txt"));
        }

        /// <summary>
        /// Test PrintReport(FileInfo file), with file is can't be created.
        /// ANE is expected.
        /// </summary>
        [Test]
        public void TestPrintReport2()
        {
            // create some processor
            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new CStyleLineCounter("C", new ExtensionMatcher("c"));

            statistics.Reporting.Reset(new DirectoryInfo("../../test_files"));

            // add processor to statistics
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            statistics.AddFileProcessor(processor3);

            // process direcotry
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/Processor"), true);

            try
            {
                statistics.PrintReport(new FileInfo("../../test_files/Reporting/extended2.result"));

                string content = UnitTestHelper.ReadFileContent("../../test_files/reporting/extended2.result");
                UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/extended2.txt",
                    "The report result is not expected.");
            }
            catch(Exception)
            {
                throw;
            }
            finally
            {
                UnitTestHelper.DeleteFile("../../test_files/reporting/extended2.result");
            }
        }
    }
}